
;**************************************************************
;**************************************************************
; LCD-Routinen
;	
; Initialisierung des LCD-Displays
InitLCD
	movlw	D'255'		; 250 ms Pause nach dem Einschalten
	movwf	loops	
	call	WAIT		

	movlw	B'00110000'	; 1
	movwf	PORTB
	bsf	PORTB, LcdE
	nop	
	bcf	PORTB, LcdE
	
	call	Wait50ms	; 50 ms Pause
	
	movlw	B'00110000'	; 2
	call	Control8Bit
	movlw	B'00110000'	; 3
	call 	Control8Bit
	movlw	B'00100000'	; 4
	call 	Control8Bit

	movlw	B'00000001'	; lschen und cusor home
	call	OutLcdControl	
	movlw	B'00101000'	; 5 function set, 4-bit  2-zeilig,  5x7
	call	OutLcdControl	
	movlw	B'00001000'	; 6 display off
	call	OutLcdControl
	movlw	B'00000110'	; 7 entry mode, increment, disable display-shift
	call	OutLcdControl
	movlw	B'00000011'	; 8 cursor home, cursor home
	call	OutLcdControl
	movlw	B'00001100'	; 9 display on, cursor off, cursor flash off
	call	OutLcdControl

; am LCD "V 2.4" ausgeben
	movlw	'V'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	' '
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'2'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'.'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'4'
	movwf	LcdDaten
	call	OutLcdDaten
	return

; ein Steuerbyte 8-bittig bertragen
Control8Bit
	movwf	PORTB
	bsf	PORTB, LcdE
	nop
	bcf	PORTB, LcdE
	call 	Wait10ms
	return

; darauf warten, da das Display bereit zur Datenannahme ist
LcdBusy
        bsf     STATUS, RP0	; make Port B4..7 input
	movlw	B'11110000'
	iorwf   TRISB, f 
        bcf     STATUS, RP0
BusyLoop		
	bcf	PortLCo, LcdRs
	bsf	PortLCo, LcdRw	; Lesen
	bsf	PortLCo, LcdE
	nop
	movf	PortLDa, w
	movwf	LcdStatus
	bcf	PortLCo, LcdE
	nop
	bsf	PortLCo, LcdE	; Enable
	nop
	bcf	PortLCo, LcdE
	btfsc	LcdStatus, 7	; teste bit 7
	goto	BusyLoop
	bcf	PortLCo, LcdRw
        bsf     STATUS, RP0	; make Port B4..7 output
	movlw	B'00001111'
	andwf   TRISB, f    
        bcf     STATUS, RP0
	return	

; ein Byte mit Steuerdaten von LcdDaten zum Display bertragen
OutLcdControl
	movwf	LcdDaten
	call	LcdBusy
	movf	LcdDaten, w
	andlw	H'F0'
	movwf	PortLDa		; Hi-teil Daten schreiben
	bsf	PortLCo, LcdE
	nop
	bcf	PortLCo, LcdE	; Disable LcdBus
	swapf	LcdDaten, w
	andlw	H'F0'
	movwf	PortLDa		; Lo-teil Daten schreiben
	bsf	PortLCo, LcdE
	nop
	bcf	PortLCo, LcdE	; Disable LcdBus
	return

; ein Datenbyte von LCDDaten zum Display bertragen
OutLcdDaten
	movwf	LcdDaten
	call	LcdBusy
	movf	LcdDaten, w
	andlw	H'F0'
	movwf	PortLDa		; Hi-teil Daten schreiben
	bsf	PortLCo, LcdRs	; Daten
	bsf	PortLCo, LcdE	; Enable LcdBus
	nop
	bcf	PortLCo, LcdE	; Disable LcdBus	
	swapf	LcdDaten, w
	andlw	H'F0'
	movwf	PortLDa		; Lo-teil Daten schreiben
	bsf	PortLCo, LcdRs	; Daten
	bsf	PortLCo, LcdE
	nop
	bcf	PortLCo, LcdE	; Disable LcdBus	
	bcf	PortLCo, LcdRs	;
	return


; Ende der Grundroutinen
;*******************************************************************************
;
;*******************************************************************************
; Kursor positionieren
LcdZeile1			; Kursor zurck
	movlw	B'00000011'	; 8 cursor home, cursor home auf Adresse 0, Display-Shift entfernen, 1.64 ms
	call	OutLcdControl
	; Kursor home
	movlw	B'10000000'	; 1. Zeile
	call	OutLcdControl
	return

LcdZeile1_1
	movlw	B'10000001'	; 1. Zeile, 2. position
	call	OutLcdControl
	return

LcdZeile1_10
	movlw	B'10001010'	; 1. Zeile, 10. position
	call	OutLcdControl
	return

LcdZeile2
	movlw	B'11000000'	; Sprung zur 2. Zeile des LCD
	call	OutLcdControl
	return	

LcdZeile2_8
	movlw	B'11001000'	; Sprung zur Mitte der 2. Zeile des LCD fuer toleranzausgabe Zaehlung
	call	OutLcdControl
	return	

LcdZeile2_7
	movlw	B'11000111'	; Sprung zur Mitte der 2. Zeile des LCD , fuer toleranz Pulslaenge
	call	OutLcdControl
	return	


;*******************************************************************************
; einzelne Zeichen/kurze Strings ausgeben

; String ' Hz' am LCD ausgeben
OutHz
	movlw	' '
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'H'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'z'
	movwf	LcdDaten
	call	OutLcdDaten
	return

OutNull
	movlw	'0'
	movwf	LcdDaten
	call	OutLcdDaten
	return
OutEinz
	movlw	'1'
	movwf	LcdDaten
	call	OutLcdDaten
	return
OutKomma
	movlw	','
	movwf	LcdDaten
	call	OutLcdDaten
	return
OutSpace
	movlw	' '
	movwf	LcdDaten
	call	OutLcdDaten
	return
OutStern
	movlw	'*'
	movwf	LcdDaten
	call	OutLcdDaten
	return
OutSharp
	movlw	'#'
	movwf	LcdDaten
	call	OutLcdDaten
	return
Out0k000
	call	OutNull
	call	OutKomma
	call	OutNull
	call	OutNull
	call	OutNull
	call	OutSpace
	return
	


;*******************************************************************************
;Eingang am LCD anzeigen
; Mode=0  :  'Amp.'
; Mode=1  :  'TTL '
; Mode=2  :  '64:1' oder '40:1'
LcdShowInput
Inp
	call	LcdZeile2

	movfw	Mode
	btfsc	STATUS,Z	; =0?
	goto	Inp0
	btfsc	Mode,0		; =1?
	goto	Inp1
Inp2
	movlw	'6'		; RA5=1: 64:1 oder RA5=0: 40:1
	movwf	LcdDaten
	btfsc	PORTA,5
	call	OutLcdDaten
	movlw	'4'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'0'
	movwf	LcdDaten
	btfss	PORTA,5
	call	OutLcdDaten
	movlw	':'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'1'
	movwf	LcdDaten
	call	OutLcdDaten
	goto	InpEnd
Inp0
	movlw	'A'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'm'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'p'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'.'
	movwf	LcdDaten
	call	OutLcdDaten
	goto	InpEnd
Inp1
	movlw	'T'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'T'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'L'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	' '
	movwf	LcdDaten
	call	OutLcdDaten
	goto	InpEnd
InpEnd
	movlw	' '
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	' '
	movwf	LcdDaten
	call	OutLcdDaten

; ein sich von Messung zu Messung ndernes Symbol
; zeigt auch bei konstantem Meergebnis Aktivitt an
; dient auch als Vorzeichen fr die Mess-Auflsung
FlipFlop
	movlw	'+'	
	incf	FlipCount,f
	btfss	FlipCount,0
	movlw	'-'
	movwf	LcdDaten
	call	OutLcdDaten

	movlw	' '
	movwf	LcdDaten
	call	OutLcdDaten

	return


FlipFlop2
	movlw	'+'	
	btfss	FlipCount,0
	movlw	'-'
	movwf	LcdDaten
	call	OutLcdDaten
	return


;*******************************************************************************
;*******************************************************************************
; Anzeige der Messtoleranz der Frequenzzaehlung
; ohne 64:1       4 x 2^PS :    4 ..   64 Hz (ohne PS 2 Hz)
; mit  64:1  64 x 4 x 2^PS :  256 .. 4096 Hz
FLcdSchoTol
	call	LcdZeile2_8

	clrf	f1
	clrf	f2
	clrf	f3
	movlw	2
	movwf	f0

	; nur fr Mode2
	; externen 64:1 oder 40:1-Vorteiler kompensieren
	btfss	Mode, 1		; Bei Mode 0 oder 1 berspringen
	goto	TolNo64
	btfsc	PORTA,5
	call	Mal64f		; 16 bit x64
	btfss	PORTA,5
	call	Mal40f		; 16 bit x40
TolNo64

	; internen Timer0-Vorteiler kompensieren
	btfsc	PreSC,3
	goto	PSNo2		; kein vorteiler verwendet
	movfw	PreSC
	movwf	Fehler		; sichern fr Berechnung der Frequenz
	incf	Fehler, f
Tolmal
	call	Mal2f		; PS kompensieren
	decfsz	Fehler,f
	goto	Tolmal
PSNo2

	; praezisionsmode
	btfsc	PpmMode		; im Praezisionsmode  vier mal genauer
	call	Durch4f		; Division: 16-Bit /4 (f)

	btfsc	PpmMode2	; im Praezisionsmode2  8 mal genauer
	call	Durch2f		; Division: 16-Bit /2 (f)

	; mindestens 1 anzeigen
	movfw	f1
	iorwf	f0, w
	skpnz
	incf	f0, f

	call	OutDez16L0	;16-bit (f1,f0) als Dez zum Lcd, 5-stellig 
	call	OutHz
	return


;*******************************************************************************
; behelfmaessige Toleranz fuer Pulsweitenmessung
PLcdSchoTol
	call	LcdZeile2_8
	; Toleranz "<<1 Hz"
	movlw	' '
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	' '
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'<'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'<'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'1'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	' '
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'H'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'z'
	movwf	LcdDaten
	call	OutLcdDaten
	return


;*******************************************************************************
; genaue toleranz fuer Pulsweitenmessung auf Grundlage des Zhlwertes
; der Zhlwert steht in i
; f darf verloren gehen
;
; |xxxxxxx10,000 Hz|
; |TTL   +10,000 Hz|
; |T +10,000 000 Hz|
;
;10 MHz Takt
;Tol		Zaehlwert	hex		freq [Hz]	ppm
;  0,000 000 1	10.000.000	989680		1,0000005	0,1
;  0,000 001	3.162.277	3040A5		3,162278	0,3
;  0,000 010	1.000.000	0F423F		10,000005	1
;  0,000 100	316.227		04D343		31,622827	3
;  0,001 000	100.000		01869F		100,000500	10
;  0,010 000	31.622		007B86		316,232766	30
;  0,100 000	10.000		00270F		1.000,050001	100
;  1,000 000	3.162		000C59		3.162,777700	300
; 10,000 000	1.000		0003E7		10.005,001250	1000
;100,000 000	316		00013B		31.672,816130	3000
;
;20 MHz Takt
;Tol		Zaehlwert	hex		freq [Hz]	ppm
;  0,000 000 1	10.000.000	989680		2		0,1
;  0,000 001	3.162.277	3040A5		6,3245566	0,3
;  0,000 010	1.000.000	0F423F		20,000005	1
;  0,000 100	316.227		04D343		63,245706	3
;  0,001 000	100.000		01869F		200,000500	10
;  0,010 000	31.622		007B86		632,471044	30
;  0,100 000	10.000		00270F		2.000,050001	100
;  1,000 000	3.162		000C59		6.325,110689	300
; 10,000 000	1.000		0003E7		20.005,00125	1000
;100,000 000	316		00013B		63.291,13924	3000
;
PLcdSchoTol2
	; Messwert i -> xw
	movfw	i0
	movwf	xw0
	movfw	i1
	movwf	xw1
	movfw	i2
	movwf	xw2
	movfw	i3
	movwf	xw3

	btfss	Pulsmode6
	goto	PLcdSchoTol2_3

	; 6 Nachkommastellen
	call	LcdZeile2
	movlw	'T'			; kann ja nur TTL sein
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	' '
	movwf	LcdDaten
	call	OutLcdDaten
	call	FlipFlop

	;< 3 Hz
	;> 0x30 40 A5 dann 0,000 001
	clrf	f3
	movlw	0x30
	movwf	f2
	movlw	0x40
	movwf	f1
	movlw	0xA5
	movwf	f0
	call	Sub32		; 0x3040A5 - xw  ; 32 bit f:=f-xw   calc=xw cnt=f
	bc	Tol0_0_001

	; 3 Hz ... 10 Hz
	;> 0x0F 42 3F dann 0,000 010
	clrf	f3
	movlw	0x0F
	movwf	f2
	movlw	0x42
	movwf	f1
	movlw	0x3F
	movwf	f0
	call	Sub32		; 0x0F423F - xw
	bc	Tol0_0_010

	; 10 Hz ... 31 Hz
	;> 0x04 D3 43 dann 0,000 100
	clrf	f3
	movlw	0x04
	movwf	f2
	movlw	0xD3
	movwf	f1
	movlw	0x43
	movwf	f0
	call	Sub32		; 0x04D343 - xw
	bc	Tol0_0_100

PLcdSchoTol2_3
	call	LcdShowInput
	call	LcdZeile2_7

	;< 100 Hz
	;> 0x01 86 9F dann 0,001
	clrf	f3
	movlw	0x01
	movwf	f2
	movlw	0x86
	movwf	f1
	movlw	0x9F
	movwf	f0
	call	Sub32		; 0x01869F - xw
	bc	Tol0_001

	; 100 Hz ... 316 Hz
	;> 0x00 7B 86 dann 0,010
	clrf	f3
	clrf	f2
	movlw	0x7B
	movwf	f1
	movlw	0x86
	movwf	f0
	call	Sub32		; 0x007B86 - xw
	bc	Tol0_010

	; 316 Hz ... 1 kHz
	;> 0x00 27 0F dann 0,100
	clrf	f3
	clrf	f2
	movlw	0x27
	movwf	f1
	movlw	0x0F
	movwf	f0
	call	Sub32		; 0x00270F - xw
	bc	Tol0_100

	; 1 kHz ... 3,162 kHz
	;> 0x00 0C 59 dann 1,000
	clrf	f3
	clrf	f2
	movlw	0x0C
	movwf	f1
	movlw	0x59
	movwf	f0
	call	Sub32		; 0x000C59 - xw
	bc	Tol1_000

	; > 3,162 kHz
Tol10
	call	OutEinz
	call	OutNull
	call	OutKomma
	call	OutNull
	call	OutNull
	call	OutNull
	goto	PLcdSchoTol000

Tol1_000
	call	OutSpace
	call	OutEinz
	call	OutKomma
	call	OutNull
	call	OutNull
	call	OutNull
	goto	PLcdSchoTol000

Tol0_100
	call	OutSpace
	call	OutNull
	call	OutKomma
	call	OutEinz
	call	OutNull
	call	OutNull
	goto	PLcdSchoTol000

Tol0_010
	call	OutSpace
	call	OutNull
	call	OutKomma
	call	OutNull
	call	OutEinz
	call	OutNull
	goto	PLcdSchoTol000

Tol0_001
	call	OutSpace
	call	OutNull
	call	OutKomma
	call	OutNull
	call	OutNull
	call	OutEinz
	goto	PLcdSchoTol000

Tol0_0_001
	call	Out0k000
	call	OutNull
	call	OutNull
	call	OutEinz
	goto	PLcdSchoTolE

Tol0_0_010
	call	Out0k000
	call	OutNull
	call	OutEinz
	call	OutNull
	goto	PLcdSchoTolE

Tol0_0_100
	call	Out0k000
	call	OutEinz
	call	OutNull
	call	OutNull
	goto	PLcdSchoTolE

PLcdSchoTol000
	; wenn noetig noch drei Nullen anhaengen
	btfss	Pulsmode6
	goto	PLcdSchoTolE
	call	OutSpace
	call	OutNull
	call	OutNull
	call	OutNull

PLcdSchoTolE
	call	OutHz	; ' Hz'
	return
	


;*******************************************************************************